open Printf
open Language

let quote_string s = "\"" ^ s ^ "\""

let rec string_of_obj = function
  | Entier i -> string_of_int i
  | Fonction (args, body) -> "(lambda (" ^ (List.fold_left (fun a b -> a ^ b ^ " ") "" args) ^ "\b) " ^ (string_of_obj body) ^ ")"
  | Chaine c -> c
  | Op_add -> "+"
  | Op_sub -> "-"
  | Op_mult -> "*"
  | Op_div -> "/"
  | Op_car -> "car"
  | Op_cdr -> "cdr"
  | Op_cwr(w) -> "c" ^ w ^ "r"
  | Op_module -> "module"
  | Op_callcc -> "calcc"
  | Op_set -> "set"
  | Op_cons -> "cons"
  | Op_quote -> "quote"
  | Op_begin -> "begin"
  | Op_is_cons -> "is_cons"
  | Op_setq -> "setq"
  | Op_let -> "let"
  | Op_lambda -> "lambda"
  | Op_eval -> "eval"
  | Op_if -> "if"
  | Op_eq_int -> "eq_int"
  | Op_print -> "print"
  | Liste [] -> "()"
  | Liste l ->
      "(" ^
      (
	let rec print_list = function
	  | [] -> ""
	  | x::s -> (string_of_obj x) ^ " " ^ (print_list s)
	in
	  print_list l ^ "\b)"
      )
  | Continuation _ -> "<cont>"
  | Exec_liste _ -> "????"    (* not supposed to ever happen *)

let quote_string_of_obj o = quote_string (string_of_obj o)

let print_obj o = printf "%s" (string_of_obj o)

let print_result = function
  | Entier i ->
      printf "- : int = %d\n" i
  | o ->
      printf "- : obj = %s\n" (string_of_obj o)
