open Language

exception Environment_panic of string

module type Environment_param =
sig
  val dynamic : bool
end

module type Environment =
sig
  type environment
  val environment_of_abs_environment : abs_environment -> environment
  val abs_environment_of_environment : environment -> abs_environment
  val link : environment -> string -> obj -> environment
  val relink : environment -> string -> obj -> environment
  val get_value : environment -> string -> obj
  val get_environment : environment -> string -> environment
  val make : unit -> environment
  val concat_environments : environment -> environment -> environment
end

module Environment (P : Environment_param) : Environment =
struct
  type environment = (string * obj * environment) list

  let abs_environment_of_environment (env : environment) : abs_environment = Obj.magic env
  let environment_of_abs_environment (env : abs_environment) : environment = Obj.magic env

  let glob = ref []

  let make () = []

  let concat_environments fst snd = fst@snd

  let rec assoc3 n = function
    | (x, a, b)::_ when x = n -> (a, b)
    | _::s -> assoc3 n s
    | [] -> raise Not_found

  let get_value loc name =
    try
      fst (assoc3 name loc)
    with
      | Not_found -> fst (assoc3 name !glob)

  let get_environment loc name =
    if P.dynamic then loc
    else
      (
	try
	  ignore (snd (assoc3 name loc));
	  loc
	with
	  | Not_found -> snd (assoc3 name !glob)
      )

  let get_linked_env env = if P.dynamic then [] else env

  let link loc name x = (name, x, (*get_linked_env loc*) [])::loc

  let rm_link name env =
    let rec aux = function
      | (n, _, _)::s when n = name -> (aux s)
      | l::s -> l::(aux s)
      | [] -> []
    in
      aux env

  let rec replace_link name x renv = function
    | (n, _, _)::s when n = name -> (name, x, renv)::s
    | t::s -> t::(replace_link name x renv s)
    | [] -> [(name, x, renv)]

  let is_loc loc name =
    try
      ignore (assoc3 name loc);
      true
    with
      | Not_found -> false

  let relink loc name x =
      glob := replace_link name x (get_linked_env loc) !glob;
      (rm_link name loc)
end
