{
  open Parser;;      (* The type token is defined in parser.mli *)
  open Language;;
  exception Eof;;
}

rule token = parse
  | [' ' '\t'] { token lexbuf }    
  | "//"       { incomment lexbuf }
  | ";;"       { STOP }
  | '\n'       { STOP (*token lexbuf*) }
  | '('        { LPAR }
  | ')'	       { RPAR }	  	
  | eof        { raise Eof }
  | '\''       { AQUOTE }
  | "nil"      { NIL }
  | "is_cons"  { IS_CONS }
  | "cons"     { CONS }
  | "car"      { CAR }
  | "cdr"      { CDR }
  | 'c' ['a' 'd']+ 'r' { let l = Lexing.lexeme lexbuf in CWR (String.sub l 1 ((String.length l)-2)) }
  | "eq_int"   { EQ_INT }
  | "quote"    { QUOTE }
  | "eval"     { EVAL }
  | "if"       { IF }
  | "set"      { SET }
  | "setq"     { SETQ }
  | "let"      { LET }
  | "lambda"   { LAMBDA }
  | "begin"    { BEGIN }
  | "print"    { PRINT }
  | "callcc"   { CALLCC }
  | "module"   { MODULE }
  | '.'        { DOT }
  | ['-']?['0'-'9']+ { INTEGER (int_of_string (Lexing.lexeme lexbuf))}
  | '+'        { PLUS }
  | '-'        { MINUS }
  | '*'        { TIMES }
  | '/'        { DIVIDES }
  | ['A'-'Z' 'a'-'z'] ['a'-'z' 'A'-'Z' '_' '0'-'9' '\'']* { ID (Lexing.lexeme lexbuf) }
  | _	       { failwith "ERREUR de SYNTAXE" }
      
and incomment = parse
  | '\n'           { token lexbuf }
  | eof            { raise Eof }
  | _              { incomment lexbuf }
