open Language
open Environment
open Kernel
open Printf
open Lexing

let _ =
  let fname = ref ""
  and dynamic = ref false
  and chan = ref stdin
  in
  Arg.parse
    [
      ("-d", Arg.Unit (fun () -> dynamic := true), "Use dynamic variables linking (instead of static)")
    ]
    (fun s -> fname := s; chan := Unix.in_channel_of_descr (Unix.openfile !fname [Unix.O_RDONLY] 0644))
    "Lisp 0.1 by Sam & Flo";
    printf "%s" ("        Lisp (" ^ (if !dynamic then "dynamic" else "static") ^ ") by Sam & Flo version 0.1\n\n");
    let module My_param =
	struct
	  let dynamic = !dynamic
	end
    in
    let module Env = Environment(My_param) in
    let module Ker = Kernel(Env) in
    let boucle = ref true in
    let res = ref (Entier (-1)) in
      while(!boucle)
      do
	try
	  if !fname = "" then (printf "# "; flush stdout);
	  let lexbuf = Lexing.from_channel !chan in
	  let result = Parser.parse Lexer.token lexbuf in
	    res := Ker.run result;
	    Display.print_result !res;
	    flush stdout;
	with
	  | Lexer.Eof -> boucle := false
	  | Kernel_panic(s) -> printf "%s" s
	  | Environment_panic(s) -> printf "%s" s
	  | Parsing.Parse_error -> printf "Parsing error\n"; flush stdout
	  | Not_found -> printf "Object not found\n"; flush stdout
	  | Stack_overflow -> printf "Error : stack overflow (maybe you do have a looping recursion ?)\n"; flush stdout
	  | _ -> printf "An unhandled error has occured\n"; flush stdout
      done;
      if !fname <> "" then Unix.close (Unix.descr_of_in_channel !chan);
      printf "\n\nBye !\n"
